import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { atDotCss } from '@iyio/at-dot-css';
import { baseLayoutCn, cn, escapeHtml, strLineCount } from '@iyio/common';
import { useCallback, useEffect, useState } from "react";
import { dt } from '../lib/lib-design-tokens';
import { LineNumbers } from './LineNumbers';
import { useShiki } from './shiki';
export const defaultCodeLineStartReg = /^(\s*)/;
export const markdownCodeLineStartReg = /^(\s*-?\s*)/;
export function CodeInput(_a) {
    var { tab = '    ', language = 'auto', value, readOnly, valueOverride, disabled, onChange, onSubmit, onElem, onTextarea, onBlur, onNewLine, tall, lineStartReg = defaultCodeLineStartReg, lineStartRegIndex = 1, lineNumbers, errors, parser, logParsed, parsingDelayMs = 700, debugParser, bottomPadding = 500 } = _a, props = __rest(_a, ["tab", "language", "value", "readOnly", "valueOverride", "disabled", "onChange", "onSubmit", "onElem", "onTextarea", "onBlur", "onNewLine", "tall", "lineStartReg", "lineStartRegIndex", "lineNumbers", "errors", "parser", "logParsed", "parsingDelayMs", "debugParser", "bottomPadding"]);
    style.root();
    const sh = useShiki(language);
    const [parsingErrors, setParsingErrors] = useState(undefined);
    const [code, setCode] = useState(null);
    const [textArea, setTextArea] = useState(null);
    useEffect(() => {
        setParsingErrors(undefined);
        if (!code || !sh || !textArea) {
            return;
        }
        code.innerHTML = sh.codeToHtml(value, { lang: language });
        let minWidth = 0;
        const codeElem = code.querySelector('code');
        if (codeElem) {
            for (let i = 0; i < codeElem.children.length; i++) {
                const item = codeElem.children.item(i);
                if (!item) {
                    continue;
                }
                const w = item.offsetWidth;
                if (w && w > minWidth) {
                    minWidth = w;
                }
            }
        }
        textArea.style.minWidth = minWidth + 'px';
        if (parser) {
            const iv = setTimeout(() => {
                try {
                    const r = parser(value, debugParser === true ? console.info : (debugParser || undefined));
                    if (logParsed) {
                        console.info('CodeInput parsing result', r);
                    }
                    setParsingErrors(r.error ? [r.error] : undefined);
                    if (r.error) {
                        console.info('⛔️ CodeInput parsing error', r.error);
                        console.info(r.error.near);
                        code.innerHTML = sh.codeToHtml(value, { lang: language, lineOptions: [
                                {
                                    line: r.error.lineNumber,
                                    classes: ['CodeInput-errorLine']
                                }
                            ] }).replace(/class="[^"]*CodeInput-errorLine[^"]*"[^>]*>/g, v => {
                            var _a, _b, _c, _d;
                            return `${v}<div class="CodeInput-error">${escapeHtml((_b = (_a = r.error) === null || _a === void 0 ? void 0 : _a.message) !== null && _b !== void 0 ? _b : '')}</div><div class="CodeInput-errorCol">${'&nbsp;'.repeat(((_d = (_c = r.error) === null || _c === void 0 ? void 0 : _c.col) !== null && _d !== void 0 ? _d : 0) - 1) + '_'}</div>`;
                        });
                    }
                }
                catch (ex) {
                    setParsingErrors([0]);
                }
            }, parsingDelayMs);
            return () => {
                clearTimeout(iv);
            };
        }
        else {
            setParsingErrors(undefined);
            return undefined;
        }
    }, [code, textArea, valueOverride, value, language, sh, parser, parsingDelayMs, logParsed, debugParser]);
    useEffect(() => {
        onElem === null || onElem === void 0 ? void 0 : onElem(code);
    }, [onElem, code]);
    useEffect(() => {
        onTextarea === null || onTextarea === void 0 ? void 0 : onTextarea(textArea);
    }, [onTextarea, textArea]);
    const onKeyDown = useCallback((e) => {
        var _a;
        if (disabled) {
            e.preventDefault();
            return;
        }
        if (!textArea) {
            return;
        }
        switch (e.key) {
            case "Tab": {
                e.preventDefault();
                const value = (textArea.value.substring(0, textArea.selectionStart) +
                    tab +
                    textArea.value.substring(textArea.selectionStart));
                const s = textArea.selectionStart + tab.length;
                textArea.value = value;
                textArea.selectionStart = s;
                textArea.selectionEnd = s;
                textArea.focus();
                onChange === null || onChange === void 0 ? void 0 : onChange(value);
                break;
            }
            case "Enter": {
                const value = (textArea.value.substring(0, textArea.selectionStart) +
                    textArea.value.substring(textArea.selectionStart));
                if (e.metaKey || e.ctrlKey) {
                    e.preventDefault();
                    const s = textArea.selectionStart;
                    textArea.value = value;
                    if (s >= 0) {
                        textArea.selectionStart = s;
                        textArea.selectionEnd = s;
                        textArea.focus();
                    }
                    onSubmit === null || onSubmit === void 0 ? void 0 : onSubmit(value);
                }
                else {
                    const s = textArea.selectionStart;
                    if (lineStartReg && s > 0 && s === textArea.selectionEnd) {
                        const i = textArea.value.lastIndexOf('\n', s - 1);
                        if (i != -1) {
                            const prevLine = textArea.value.substring(i + 1, s);
                            const lineStart = (_a = lineStartReg.exec(prevLine)) === null || _a === void 0 ? void 0 : _a[lineStartRegIndex];
                            if (lineStart) {
                                e.preventDefault();
                                textArea.value = textArea.value.substring(0, s) + '\n' + lineStart + textArea.value.substring(s);
                                textArea.selectionStart = s + lineStart.length + 1;
                                textArea.selectionEnd = s + lineStart.length + 1;
                                onChange === null || onChange === void 0 ? void 0 : onChange(textArea.value);
                                textArea.focus();
                            }
                        }
                    }
                    onNewLine === null || onNewLine === void 0 ? void 0 : onNewLine(value);
                }
                break;
            }
        }
    }, [textArea, tab, onChange, onSubmit, onNewLine, disabled, lineStartReg, lineStartRegIndex]);
    const pad = _jsx("div", { className: "CodeInput-bottomMargin", style: { height: bottomPadding } });
    return (_jsxs("div", { className: cn("CodeInput", { tall, disabled, readOnly, lineNumbers: lineNumbers !== undefined }, baseLayoutCn(props)), children: [lineNumbers && _jsx(LineNumbers, { count: strLineCount(value), errors: errors !== null && errors !== void 0 ? errors : parsingErrors, pad: pad }), _jsx("div", { className: "CodeInput-content", children: _jsxs("div", { children: [_jsx("div", { ref: setCode, className: cn("no-select", language === 'auto' ? undefined : `language-${language}`) }, language), _jsx("textarea", { spellCheck: false, ref: setTextArea, onKeyDown: onKeyDown, onChange: e => onChange === null || onChange === void 0 ? void 0 : onChange(e.target.value), onBlur: e => onBlur === null || onBlur === void 0 ? void 0 : onBlur(e.target.value), value: value, readOnly: readOnly }), pad] }) })] }));
}
const style = atDotCss({ name: 'CodeInput', css: `
    .CodeInput{
        display:flex;
        border-radius:8px;
        transition:opacity 0.1s ease-in-out;
        background:#1E1E1E;
    }
    .CodeInput textarea{
        all:unset;
    }
    .CodeInput.tall{
        min-height:200px;
    }
    .CodeInput.disabled{
        opacity:0.4;
    }
    .CodeInput.disabled textarea{
        opacity:0;
    }
    .CodeInput code, .CodeInput textarea{
        margin:0 !important;
        padding: 0 !important;
        display:block;
        white-space:pre;
        font-family:Courier !important;
        font-size:${dt().codeFontSize}px !important;
        line-height:${dt().codeLineHeight}px !important;
        min-height:${dt().codeLineHeight}px;
    }
    .CodeInput code *{
        font-size:${dt().codeFontSize}px !important;
        line-height:${dt().codeLineHeight}px !important;
    }
    .CodeInput pre{
        margin:0 !important;
        padding:0 !important;
        display:block;
    }
    .CodeInput pre,.CodeInput code{
        background: none !important;
    }
    .CodeInput textarea{
        position:absolute;
        left:0;
        top:0;
        width:100%;
        height:100%;
        overflow:hidden;
        overflow:clip;
        border:none;
        background:none;
        resize:none;
        color:transparent;
        caret-color: #ffffffcc;
        outline:none;
        outline-width:0;
        cursor:text;
    }
    .CodeInput.readOnly textarea{
        pointer-events:none;
    }
    .CodeInput .no-select{
        pointer-events:none;
    }
    .CodeInput.readOnly .no-select{
        pointer-events:auto;
    }
    .CodeInput-content{
        display:flex;
        flex-direction:column;
        overflow-x:auto;
        flex:1;
    }
    .CodeInput-content > div{
        position:relative;
    }
    .CodeInput.lineNumbers .CodeInput-content > div{
        margin-left:0.5rem !important;
    }

    .CodeInput-errorLine{
        position:relative;
    }
    .CodeInput-errorLine > .CodeInput-error{
        position:absolute;
        left:calc( 100% + 1rem);
        font-size:12px !important;
        line-height:12px !important;
        display:flex;
        align-items:center;
        background-color:#ff000033;
        top:-0.2rem;
        bottom:-0.2rem;
        padding:0 0.4rem;
        border-radius:4px;
    }
    .CodeInput-errorCol{
        position:absolute;
        left:0;
        top:0;
        color:#ff000088;
    }

` });
//# sourceMappingURL=CodeInput.js.map